<html>
<head>
  <link rel="stylesheet" href="https://wikimedia-ui.wmflabs.org/MW/mediawiki.min.css">
</head>
<body>
  <style type="text/css">
    html {
      background: #eee;
    }
    body {
      max-width: 50em;
      margin: 0px auto;
      padding: 1em 2em;
      background: #fff;
    }
  </style>
  <img src="/static/images/ores_logo.svg"
       style="float: right; padding: 0px 0px 1em 1em" />
  <p>
    The Objective Revision Evaluation Service (ORES) is a web service running
    in Wikimedia Labs that provides machine learning as a service for Wikimedia
    Projects. The system is designed to help automate critical wiki-work --
    for example, vandalism detection and removal. This service is developed as
    part of the
    <a href="https://meta.wikimedia.org/wiki/Research:Revision_scoring_as_a_service">
      Revision scoring as a service</a> research project.
  </p>

  <h2>Scores API</h2>
  <p>
    ORES is intended to be used as a source of information by tool developers.
    To access ORES, scores, a simple RESTful API is provided.  There are two
    versions of the scoring API that differ slightly in their
    behavior.  Version 2 provides access to model info in a scoring request.
    Version 1 is preserved for backwards compatibility.
  </p>
  <dl>
    <dt>Version 2 (<a href="/v2/">docs</a>)</dt>
    <dd style="padding-left: 1em;">
      The current end recommended version of the API.<br/>
      <i>
        Accessible via <a href="/v2/scores/">/v2/scores/</a>.
      </i>
    </dd>
    <dt>Version 1 (<a href="/v1/">docs</a>)</dt>
    <dd style="padding-left: 1em;">
      Preserved for backwards compatibility -- this version of the API
      implements ORES original behavior.  <br/>
      <i>
        Accessible via
        <a href="/scores/">/scores/</a> and
        <a href="/v1/scores/">/v1/scores/</a>.
      </i>
    </dd>
  </dl>

  <h2>Reference scoring user interface</h2>
  <p>
    ORES provides a simple user interface for obtaining scores.
    See <a href="/ui/">/ui/</a>.
  </p>

  <h2>Documentation</h2>
  <p>
    Read about the service and how to use it here:
    <a href="https://meta.wikimedia.org/wiki/Objective_Revision_Evaluation_Service">
      https://meta.wikimedia.org/wiki/Objective_Revision_Evaluation_Service</a>
  </p>

  <h2>Privacy policy</h2>
  <p>
    See the Wikimedia policy here: <a href="https://wikimediafoundation.org/wiki/Privacy_policy">https://wikimediafoundation.org/wiki/Privacy_policy</a>
  </p>

  <hr style="border: 0.5px solid silver"/>

  <div style="text-align: center;">
    Hosted on <a href="https://wikitech.wikimedia.org">Wikimedia Labs</a> &amp; written in <a href="https://python.org">Python 3</a>
  </div>
</body>
</html>
