<html>
<head>
  <link rel="stylesheet" href="https://wikimedia-ui.wmflabs.org/MW/mediawiki.min.css">
</head>
<body>
  <style type="text/css">
    body {
      max-width: 50em;
      margin: 1em auto;
    }
  </style>
  <p>
    This is the base of the "v2" interface for ORES' API.  See
    <a href="/v2/scores/">/v2/scores</a> for a list of supported Wikis.
  </p>
  <p>
    For more information, see <a href="//meta.wikimedia.org/wiki/ORES">meta.wikimedia.org/wiki/ORES</a>.
  </p>
</body>
</html>
